import { fileURLToPath } from 'url';
import { defu } from 'defu';
import { useLogger, defineNuxtModule, createResolver, addImports, addComponent, addServerHandler, addTemplate } from '@nuxt/kit';
import { existsSync } from 'node:fs';

const DEFAULT_CACHE_TAG_INVALIDATION_DELAY = 6e4;
const DEFAULT_API_PREFIX = "/__nuxt_multi_cache";
const DEFAULT_CDN_CONTROL_HEADER = "Surrogate-Control";
const DEFAULT_CDN_TAG_HEADER = "Cache-Tag";
const defaultOptions = {
  api: {
    prefix: DEFAULT_API_PREFIX,
    cacheTagInvalidationDelay: DEFAULT_CACHE_TAG_INVALIDATION_DELAY
  }
};

const logger = useLogger("nuxt-multi-cache");
const fileExists = (path, extensions = ["js", "ts"]) => {
  if (!path) {
    return null;
  } else if (existsSync(path)) {
    return path;
  }
  const extension = extensions.find(
    (extension2) => existsSync(`${path}.${extension2}`)
  );
  return extension ? `${path}.${extension}` : null;
};

function checkObsoleteOptions(options) {
  const caches = ["component", "data", "route"];
  caches.forEach((v) => {
    if (options[v] && options[v].storage) {
      logger.error(
        `The "storage" option on the cache configuration has been moved to the server options file.
Learn more: https://nuxt-multi-cache.dulnan.net/overview/server-options`
      );
    }
  });
  if (typeof options.api?.authorization === "function") {
    logger.error(
      `The "api.authorization" option to use a custom callback has been moved to the server options file.
Learn more: https://nuxt-multi-cache.dulnan.net/overview/server-options`
    );
  }
  if (options.enabledForRequest) {
    logger.error(
      `The "enabledForRequest" option has been moved to the server options file.
Learn more: https://nuxt-multi-cache.dulnan.net/overview/server-options`
    );
  }
}
const module = defineNuxtModule({
  meta: {
    name: "nuxt-multi-cache",
    configKey: "multiCache",
    version: "3.0.0",
    compatibility: {
      nuxt: "^3.3.0"
    }
  },
  defaults: defaultOptions,
  async setup(passedOptions, nuxt) {
    const options = defu({}, passedOptions, {});
    checkObsoleteOptions(options);
    const { resolve } = createResolver(import.meta.url);
    const rootDir = nuxt.options.rootDir;
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir);
    nuxt.options.runtimeConfig.multiCache = {
      rootDir,
      cdn: {
        cacheControlHeader: options.cdn?.cacheControlHeader || DEFAULT_CDN_CONTROL_HEADER,
        cacheTagHeader: options.cdn?.cacheTagHeader || DEFAULT_CDN_TAG_HEADER
      },
      component: !!options.component?.enabled,
      data: !!options.data?.enabled,
      route: !!options.route?.enabled,
      api: {
        enabled: !!options.api?.enabled,
        prefix: options.api?.prefix || "",
        cacheTagInvalidationDelay: options.api?.cacheTagInvalidationDelay,
        authorizationToken: typeof options.api?.authorization === "string" ? options.api.authorization : "",
        authorizationDisabled: options.api?.authorization === false
      }
    };
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.externals = defu(
        typeof nitroConfig.externals === "object" ? nitroConfig.externals : {},
        {
          inline: [resolve("./runtime")]
        }
      );
    });
    if (options.data) {
      addImports({
        from: resolve("./runtime/composables/useDataCache"),
        name: "useDataCache"
      });
    }
    if (options.route) {
      addImports({
        from: resolve("./runtime/composables/useRouteCache"),
        name: "useRouteCache"
      });
    }
    if (options.cdn) {
      addImports({
        from: resolve("./runtime/composables/useCDNHeaders"),
        name: "useCDNHeaders"
      });
    }
    nuxt.options.alias["#nuxt-multi-cache/composables"] = resolve(
      "runtime/composables/index"
    );
    if (options.component) {
      await addComponent({
        filePath: resolve("./runtime/components/RenderCacheable/index"),
        name: "RenderCacheable",
        global: true
      });
    }
    if (options.component || options.route || options.data) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/cacheContext"),
        middleware: true
      });
    }
    if (options.cdn?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/cdnHeaders"),
        middleware: true
      });
    }
    if (options.route?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/serveCachedRoute")
      });
    }
    if (options.cdn?.enabled || options.route?.enabled) {
      addServerHandler({
        handler: resolve("./runtime/serverHandler/responseSend")
      });
    }
    const extensions = ["js", "mjs", "ts"];
    const resolvedPath = "~/app/multiCache.serverOptions".replace(/^(~~|@@)/, nuxt.options.rootDir).replace(/^(~|@)/, nuxt.options.srcDir);
    const template = (() => {
      const resolvedFilename = `multiCache.serverOptions.ts`;
      const maybeUserFile = fileExists(resolvedPath, extensions);
      if (maybeUserFile) {
        return addTemplate({
          filename: resolvedFilename,
          write: true,
          getContents: () => `export { default } from '${resolvedPath}'`
        });
      }
      return addTemplate({
        filename: resolvedFilename,
        write: true,
        getContents: () => "export default {}"
      });
    })();
    nuxt.options.nitro.externals = nuxt.options.nitro.externals || {};
    nuxt.options.nitro.externals.inline = nuxt.options.nitro.externals.inline || [];
    nuxt.options.nitro.externals.inline.push(template.dst);
    nuxt.options.alias["#multi-cache-server-options"] = template.dst;
    if (options.api?.enabled) {
      const apiPrefix = options.api.prefix;
      const prefix = (path) => apiPrefix + "/" + path;
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeAll"),
        method: "post",
        route: prefix("purge/all")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeTags"),
        method: "post",
        route: prefix("purge/tags")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/purgeItem"),
        method: "post",
        route: prefix("purge/:cacheName")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/stats"),
        method: "get",
        route: prefix("stats/:cacheName")
      });
      addServerHandler({
        handler: resolve("./runtime/serverHandler/api/inspectItem"),
        method: "get",
        route: prefix("inspect/:cacheName")
      });
    }
  }
});
function defineMultiCacheOptions(options) {
  return options;
}

export { module as default, defineMultiCacheOptions };
