import {
  defineComponent,
  useSSRContext,
  useSlots,
  getCurrentInstance,
  h
} from "vue";
import { useNuxtApp } from "#app";
import {
  getExpiresValue,
  getMultiCacheContext,
  getCacheKeyWithPrefix
} from "./../../helpers/server";
import { getCacheKey, getCachedComponent, renderSlot } from "./helpers/index.mjs";
export default defineComponent({
  name: "RenderCacheable",
  props: {
    tag: {
      type: String,
      default: "div"
    },
    noCache: {
      type: Boolean,
      default: false
    },
    cacheKey: {
      type: String,
      default: ""
    },
    cacheTags: {
      type: Array,
      default: () => []
    },
    maxAge: {
      type: Number,
      default: 0
    },
    asyncDataKeys: {
      type: Array,
      default: () => []
    }
  },
  async setup(props) {
    const slots = useSlots();
    if (!slots.default) {
      return () => "";
    }
    const defaultSlot = slots.default();
    const first = defaultSlot[0];
    if (process.server && !props.noCache) {
      const cacheKey = getCacheKey(props, first);
      if (!cacheKey) {
        return () => h(props.tag, slots.default());
      }
      const currentInstance = getCurrentInstance();
      const ssrContext = useSSRContext();
      if (!ssrContext) {
        console.log("Failed to get SSR context.");
        return () => h(props.tag, slots.default());
      }
      const getOrCreateCachedComponent = async () => {
        if (!currentInstance?.parent) {
          console.log("Failed to get parent component in Cacheable component.");
          return;
        }
        const nuxtApp = useNuxtApp();
        const multiCache = getMultiCacheContext(ssrContext.event);
        if (!multiCache?.component) {
          console.log("Component cache is disabled.");
          return;
        }
        const fullCacheKey = getCacheKeyWithPrefix(cacheKey, ssrContext.event);
        const cached = await getCachedComponent(
          multiCache.component,
          fullCacheKey
        );
        if (cached) {
          const { data: data2, payload, expires } = cached;
          if (expires) {
            const now = Date.now() / 1e3;
            if (now >= expires) {
              return;
            }
          }
          if (payload) {
            Object.keys(payload).forEach((key) => {
              nuxtApp.payload.data[key] = payload[key];
            });
          }
          return data2;
        }
        const data = await renderSlot(slots, currentInstance.parent);
        try {
          const cacheTags = props.cacheTags;
          if (props.asyncDataKeys.length || cacheTags.length || props.maxAge) {
            const payload = props.asyncDataKeys.reduce((acc, key) => {
              acc[key] = nuxtApp.payload.data[key];
              return acc;
            }, {});
            const item = { payload, data, cacheTags };
            if (props.maxAge) {
              item.expires = getExpiresValue(props.maxAge);
            }
            multiCache.component.setItem(fullCacheKey, item);
          } else {
            multiCache.component.setItem(fullCacheKey, data);
          }
        } catch (e) {
          if (e instanceof Error) {
            console.error(e.message);
          }
        }
        return data;
      };
      try {
        const cachedMarkup = await getOrCreateCachedComponent();
        if (cachedMarkup) {
          return () => h(props.tag, {
            innerHTML: cachedMarkup
          });
        }
      } catch (e) {
        if (e instanceof Error) {
          console.error(e.message);
        }
      }
    }
    return () => h(props.tag, slots.default());
  }
});
